/*
 * MORFEO Project
 * http://www.morfeo-project.org
 *
 * Component: TIDorbJTest
 * Programming Language: Java
 *
 * File: $Source: TestClient.java $
 * Version: $Revision: 1.0 $
 * Date: $Date: 10-ago-2006 $
 * Last modified by: $Author: irenka.redondo $
 *
 * (C) Copyright 2004 Telefnica Investigacin y Desarrollo
 *     S.A.Unipersonal (Telefnica I+D)
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 */

import java.util.Properties;
import java.io.*;

import Test.*;
//import org.omg.CORBA.Exception;
import org.omg.CORBA.SystemException;

import org.omg.CORBA_2_3.ORB;

public class TestClient {
    public static void main(String args[])
    {
    	try {
            // Create and initialize orb
            Properties props = new Properties();
            props.setProperty("org.omg.CORBA.ORBClass", 
                              "es.tid.TIDorbj.core.TIDORB");
            props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                              "es.tid.TIDorbj.core.SingletonORB");
            props.setProperty("es.tid.TIDorbj.trace.level", "3");
            props.setProperty("es.tid.TIDorbj.trace.file", "client.log");

//             props.setProperty("org.omg.CORBA.ORBClass",
//                               "org.jacorb.orb.ORB");
//             props.setProperty("org.omg.CORBA.ORBSingletonClass",
//                               "org.jacorb.orb.ORBSingleton");

            //org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,props);
            org.omg.CORBA_2_3.ORB orb = (org.omg.CORBA_2_3.ORB) ORB.init(args, props);


            // Registro de la factora de Valuetypes
            org.omg.CORBA.portable.ValueFactory simple_factory = new MySimpleValueTypeInitImpl();
            String rep_id_simple = Test.SimpleValueTypeHelper.id();
            orb.register_value_factory(rep_id_simple, simple_factory);

            org.omg.CORBA.portable.ValueFactory complex_factory = new MyComplexValueTypeInitImpl();
            String rep_id_complex = Test.ComplexValueTypeHelper.id();
            orb.register_value_factory(rep_id_complex, complex_factory);


            // Get ior
            String ior_str = null;
            String dbior = "Test.ior";
            File ior_fl = new File(dbior);
        
            if (ior_fl.exists()) { //ior in file
                try {
                    BufferedReader br = new BufferedReader(new FileReader(ior_fl));
                    ior_str = br.readLine();
                    br.close();
                } catch (java.io.IOException e) {
                    System.err.println("Error reading file " + dbior);
                    System.exit(1);
                }
            }
            else { //ior in standard input
                try {
                    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
                    ior_str = br.readLine();
                    br.close();
                } catch (java.io.IOException e) {
                    System.err.println("Error standard input");
                    System.exit(1);
                }
            }	

            // Get object reference
            org.omg.CORBA.Object obj = orb.string_to_object(ior_str);
            ValueTypeTester refTester = ValueTypeTesterHelper.narrow(obj);

            String message = "Hola mundo";
            short mynum = 1;


            try {
                System.out.println("[Client] invoking op1 ...");
                SimpleValueType value = new MySimpleValueTypeImpl();
                value.num = mynum;
                refTester.op1(value);
            } catch (SystemException e) {
                System.err.println("Main: CORBA exception " + e.getMessage());
                //e.printStackTrace();
            }


            try {
                System.out.println("[Client] invoking op2 ...");
                SimpleValueTypeHolder value_out = new SimpleValueTypeHolder();
                refTester.op2(value_out);
                //System.out.println("[Client] value_res.num = " + value_out.value.num );
            } catch (SystemException e) {
                System.err.println("Main: CORBA exception " + e.getMessage());
                //e.printStackTrace();
            }


            try {
                System.out.println("[Client] invoking op3 ...");
                mynum = 3;
                SimpleValueTypeHolder value_inout = 
                    new SimpleValueTypeHolder(new MySimpleValueTypeImpl(mynum));
                refTester.op3(value_inout);
                System.out.println("[Client] value_inout.num = " + value_inout.value.num );
            } catch (SystemException e) {
                System.err.println("Main: CORBA exception " + e.getMessage());
                //e.printStackTrace();
            }


            try {
                System.out.println("[Client] invoking op4 ...");
                SimpleValueType value_res = refTester.op4();
                System.out.println("[Client] value_res.num = " + value_res.num );
            } catch (SystemException e) {
                System.err.println("Main: CORBA exception " + e.getMessage());
                //e.printStackTrace();
            }


            System.out.println("[Client] Destroying ORB ...");
            orb.destroy();
            
            //System.exit(0);
    	}
    	catch (Exception e) {
            System.err.println("Main: Unexpected exception " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
