import Test.*;

import org.omg.PortableServer.ForwardRequest;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.Servant;
import org.omg.PortableServer.ServantActivatorOperations;
import org.omg.CORBA.*;

import java.lang.Byte;
import java.lang.Integer;

public class ThermometerActivatorImpl implements ServantActivatorOperations {

    private int m_thermometers;

    public ThermometerActivatorImpl(int max) {
        m_thermometers = max;
    }

    public static int byteArrayToInt(byte[] b, int offset) {
        int value = 0;
        for (int i = 0; i < 4; i++) {
            int shift = (4 - 1 - i) * 8;
            value += (b[i + offset] & 0x000000FF) << shift;
        }
        return value;
    }


    public Servant incarnate(byte[] oid, POA adapter) 
        throws ForwardRequest 
    {
        System.out.println("ThermometerActivator_impl::incarnate() ");
        String cadena_oid;

        //
        // Check if Id of object is valid
        //
        try {
            cadena_oid = new String(oid); 
        } catch (Exception e) {
            System.err.println("ThermometerActivatorImpl oid.toString() exception " + e.getMessage());
            throw new org.omg.CORBA.OBJECT_NOT_EXIST();
        }

        //
        // Get element number from cadena_oid
        //
        int element = 0;
        try {
            element = Integer.valueOf(cadena_oid).intValue();
        } catch (Exception e) {
            System.err.println("ThermometerActivatorImpl cadena_oid.intValue() exception " + 
                               e.getMessage() + " cadena_oid " + cadena_oid);
            throw new org.omg.CORBA.OBJECT_NOT_EXIST();
        }

        if (element > m_thermometers) {
            System.err.println("ThermometerActivatorImpl element > m_thermometers ");
            throw new org.omg.CORBA.OBJECT_NOT_EXIST();
        }
        

        //
        // Create thermomether
        //
        Servant servant = new ThermometerImpl(element);
        return servant;
    } // incarnate

    public void etherealize(byte[] oid, POA adapter, Servant serv, 
                            boolean cleanup_in_progress, boolean remaining_activations) 
    {
        System.out.println("[server] ThermometerActivator_impl::etherealize() ");
        // TODO: if (!remaining_activations)
    } // etherealize
    

} // class ThermometerActivatorImpl
