import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.Properties;

import org.omg.CORBA.ORB;
import org.omg.PortableServer.*;


public class server_ServantActivator {
    
    public static void main(String[] args) {
        
        org.omg.CORBA_2_3.ORB orb = 
            (org.omg.CORBA_2_3.ORB) org.omg.CORBA.ORB.init(args, System.getProperties());

        try {
            
            org.omg.CORBA.Object poa_ref = 
                orb.resolve_initial_references("RootPOA");
            POA rootPOA = POAHelper.narrow(poa_ref);
            POAManager rootPOA_mgr = rootPOA.the_POAManager();

            /*
             * Create and activate a poa for thermometers
             *
             */
            org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[4];
            POA thermometers_poa = null;
            POAManager thermometers_poa_mgr = null;
            try{
                policies[0] = new es.tid.TIDorbj.core.poa.policies.IdAssignmentPolicyImpl(
                                              IdAssignmentPolicyValue.USER_ID);
                policies[1] = new es.tid.TIDorbj.core.poa.policies.RequestProcessingPolicyImpl(
                                              RequestProcessingPolicyValue.USE_SERVANT_MANAGER);
                policies[2] = new es.tid.TIDorbj.core.poa.policies.ServantRetentionPolicyImpl(
                                              ServantRetentionPolicyValue.RETAIN);
                policies[3] = new es.tid.TIDorbj.core.poa.policies.LifespanPolicyImpl(
                                              LifespanPolicyValue.PERSISTENT);
                
                thermometers_poa = rootPOA.create_POA("Thermometers_POA", null, policies);
                thermometers_poa_mgr = thermometers_poa.the_POAManager();

            } catch (Exception ex) {
                System.err.println("[servert_ServantActivator] Exception creating poa ");
            }

            /*
             * Create an Activator
             */
            ThermometerActivatorImpl activator = new ThermometerActivatorImpl(10);
            ServantActivatorLocalTie activator_tie = 
    	  	new ServantActivatorLocalTie(activator);

            thermometers_poa.set_servant_manager(activator_tie);


            /*
             * Create and activate a Controller object
             */
            org.omg.CORBA.Policy[] policies2 = new org.omg.CORBA.Policy[2];
            POA controller_poa = null;
            try{
                policies2[0] = new es.tid.TIDorbj.core.poa.policies.LifespanPolicyImpl(
                                              LifespanPolicyValue.PERSISTENT);
                policies2[1] = new es.tid.TIDorbj.core.poa.policies.IdAssignmentPolicyImpl(
                                              IdAssignmentPolicyValue.USER_ID);               
                controller_poa = rootPOA.create_POA("Controller_POA", 
                                                      rootPOA_mgr, 
                                                      policies2);
            } catch (Exception ex) {
                System.err.println("[servert_ServantActivator] Exception creating poa ");
            }

            ControllerImpl controller = new ControllerImpl(thermometers_poa, orb);
            byte[] oid_controller = "Controller".getBytes();
            controller_poa.activate_object_with_id(oid_controller, controller);

            
            /*
             * Write reference to file
             */
            org.omg.CORBA.Object obj_controller = controller_poa.id_to_reference(oid_controller);
            String reference = orb.object_to_string(obj_controller);
            String dbior = "Tester.ior";
            File ior_fl = new File(dbior);
            try {
                BufferedWriter bw = new BufferedWriter(new FileWriter(ior_fl));
                bw.write(reference, 0, reference.length());
                bw.flush();
                bw.close();
            } catch (java.io.IOException e) {
                System.err.println("[Server] Error writing file " + dbior);
                System.exit(0);
            }

            thermometers_poa_mgr.activate();

            rootPOA_mgr.activate();

            System.err.println("[server_ServantActivator] running... ");
            orb.run();
            orb.destroy();
            
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        
    } // main method
    
} // server_ServantActivator class
