/*
 * MORFEO Project
 * http://www.morfeo-project.org
 *
 * Component: TIDorbJTest
 * Programming Language: Java
 *
 * File: $Source: TestClient.java $
 * Version: $Revision: 1.0 $
 * Date: $Date: 10-ago-2006 $
 * Last modified by: $Author: avega $
 *
 * (C) Copyright 2004 Telefnica Investigacin y Desarrollo
 *     S.A.Unipersonal (Telefnica I+D)
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 */

import java.util.Properties;
import java.io.*;

import Test.*;

import org.omg.CORBA.*;
import org.omg.Messaging.*;
import org.omg.PortableServer.*;

import org.omg.CORBA_2_3.ORB;

public class TestClient {
    public static void main(String args[])
    {
    	try {
            // Create and initialize orb
            Properties props = new Properties();
            props.setProperty("org.omg.CORBA.ORBClass", 
                              "es.tid.TIDorbj.core.TIDORB");
            props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                              "es.tid.TIDorbj.core.SingletonORB");
            props.setProperty("es.tid.TIDorbj.trace.level", "3");
            props.setProperty("es.tid.TIDorbj.trace.file", "client.log");

//             props.setProperty("org.omg.CORBA.ORBClass",
//                               "org.jacorb.orb.ORB");
//             props.setProperty("org.omg.CORBA.ORBSingletonClass",
//                               "org.jacorb.orb.ORBSingleton");

            //org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,props);
            org.omg.CORBA_2_3.ORB orb = (org.omg.CORBA_2_3.ORB) ORB.init(args, props);



            // Get ior
            String ior_str = null;
            String dbior = "Tester.ior";
            File ior_fl = new File(dbior);
        
            if (ior_fl.exists()) { //ior in file
                try {
                    BufferedReader br = new BufferedReader(new FileReader(ior_fl));
                    ior_str = br.readLine();
                    br.close();
                } catch (java.io.IOException e) {
                    System.err.println("Error reading file " + dbior);
                    System.exit(1);
                }
            }
            else { //ior in standard input
                try {
                    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
                    ior_str = br.readLine();
                    br.close();
                } catch (java.io.IOException e) {
                    System.err.println("Error standard input");
                    System.exit(1);
                }
            }	
            

            org.omg.CORBA.Object obj = null;
            POA myPOA = null;


            // Obtain ORBPolicyManager
            PolicyManager policy_manager = null;
            try{
                System.out.println("[Client] Obtaining ORBPolicyManager ... ");
                obj = orb.resolve_initial_references ("ORBPolicyManager");
                policy_manager = PolicyManagerHelper.narrow(obj);
                System.out.println("[Client] ... OK ");
            } catch (Exception ex) {
                System.err.println("[Client] Exception " + ex.getMessage());
            }


            // Obtain PolicyCurrent
            PolicyCurrent policy_current = null;
            try{
                System.out.println("[Client] Obtaining PolicyCurrent ... ");
                obj = orb.resolve_initial_references ("PolicyCurrent");
                policy_current = PolicyCurrentHelper.narrow(obj);
                System.out.println("[Client] ... OK ");
            } catch (Exception ex) {
                System.err.println("[Client] Exception " + ex.getMessage());
            }


            // Client side policies
            Any rebind_any = orb.create_any();
            RebindModeHelper.insert(rebind_any, NO_RECONNECT.value);


            // Set the policies
            Policy[] policies = new Policy[1];
            try{
                System.out.println("[Client] Setting policies ... ");
                policies[0] = orb.create_policy(REBIND_POLICY_TYPE.value,
                                                rebind_any);
                System.out.println("[Client] ... OK ");
            } catch (Exception ex) {
                System.err.println("[Client] Exception " + ex.getMessage());
            }


            try{
                System.out.println("[Client] policy_manager.set_policy_overrides ... ");
                policy_manager.set_policy_overrides(policies, 
                                                    SetOverrideType.ADD_OVERRIDE);
                System.out.println("[Client] ... OK ");
            } catch (Exception ex) {
                System.err.println("[Client] Exception " + ex.getMessage());
            }


            try{
                System.out.println("[Client] policy_current.set_policy_overrides ... ");
                policy_current.set_policy_overrides(policies,
                                                    SetOverrideType.ADD_OVERRIDE);
                System.out.println("[Client] ... OK ");
            } catch (Exception ex) {
                System.err.println("[Client] Exception " + ex.getMessage());
            }


            // Get object reference
            org.omg.CORBA.Object obj2 = orb.string_to_object(ior_str);


            try{
                System.out.println("[Client] ref._set_policy_override... ");
                //refTester._set_policy_override(policies, 
                obj2._set_policy_override(policies, 
                                               SetOverrideType.ADD_OVERRIDE);
                System.out.println("[Client] ... OK ");
            } catch (Exception ex) {
                System.err.println("[Client] Exception " + ex.getMessage());
            }



            Tester refTester = TesterHelper.narrow(obj2);


            String message = "Hola mundo";
            String response = refTester.echo(message);
            System.out.println("[Client] echo(" + message + ") = " + response);



            System.out.println("[Client] Destroying ORB ...");
            orb.destroy();
            
            //System.exit(0);
    	}
    	catch (Exception e) {
            System.err.println("Main: Unexpected exception " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
