/*
 * MORFEO Project
 * http://www.morfeo-project.org
 *
 * Component: TIDorbJTest
 * Programming Language: Java
 *
 * File: $Source: TestServer.java $
 * Version: $Revision: 1.0 $
 * Date: $Date: 10-ago-2006 $
 * Last modified by: $Author: avega $
 *
 * (C) Copyright 2004 Telefnica Investigacin y Desarrollo
 *     S.A.Unipersonal (Telefnica I+D)
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 */

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.Properties;

import org.omg.CORBA_2_3.ORB;

import org.omg.PortableServer.*;
import org.omg.CORBA.*;
import org.omg.Messaging.*;

public class TestServer {
    public static void main(String[] args) {
        try {
            // Create and initialize orb
            Properties props = new Properties();
            
            props.setProperty("org.omg.CORBA.ORBClass", 
                              "es.tid.TIDorbj.core.TIDORB");
            props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                              "es.tid.TIDorbj.core.SingletonORB");
            props.setProperty("es.tid.TIDorbj.trace.level", "4");
            props.setProperty("es.tid.TIDorbj.trace.file", "server.log");

//             props.setProperty("org.omg.CORBA.ORBClass",
//                               "org.jacorb.orb.ORB");
//             props.setProperty("org.omg.CORBA.ORBSingletonClass",
//                               "org.jacorb.orb.ORBSingleton");


            //ORB orb = ORB.init(args, props);
            org.omg.CORBA_2_3.ORB orb = (org.omg.CORBA_2_3.ORB) ORB.init(args, props);


            // Get rootPOA reference and activate POAManager
            POA rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
            org.omg.PortableServer.POAManager rootPOA_mgr = rootPOA.the_POAManager();
            rootPOA_mgr.activate();
            

            /*
             * Server side policies: 
             */ 
            //Ordering ordering = ORDER_TEMPORAL.value;
            //RoutingType routing = ROUTE_FORWARD.value;
            //RebindMode rebind = TRANSPARENT.value;
            PriorityRange request_priority = new PriorityRange();
            request_priority.min = 1;
            request_priority.max = 255;
            PriorityRange reply_priority = new PriorityRange();
            reply_priority.min = 1;
            reply_priority.max = 255;
            
            Any ordering_any = orb.create_any();
            Any routing_any = orb.create_any();
            Any rebind_any = orb.create_any();
            Any request_priority_any = orb.create_any();
            Any reply_priority_any = orb.create_any();

            
            OrderingHelper.insert(ordering_any, ORDER_TEMPORAL.value);
            RoutingTypeHelper.insert(routing_any, ROUTE_FORWARD.value);
            RebindModeHelper.insert(rebind_any, TRANSPARENT.value);
            PriorityRangeHelper.insert(request_priority_any, request_priority);
            PriorityRangeHelper.insert(reply_priority_any, reply_priority);
            
            
            /*
             * Create and activate a poa
             *
             */
            Policy[] policies = new Policy[2];
            POAManager myPOA_mgr = null;
            POA myPOA = null;

            try{
                policies[0] = orb.create_policy(QUEUE_ORDER_POLICY_TYPE.value,
                                                ordering_any);
                policies[1] = orb.create_policy(REQUEST_PRIORITY_POLICY_TYPE.value,
                                                request_priority_any);
                myPOA = rootPOA.create_POA("My_POA", null, policies);
            
                myPOA_mgr = myPOA.the_POAManager();
            } catch (Exception ex) {
                System.err.println("[Server] creating poa ");
            }
            myPOA_mgr.activate();
            

            // Create servant
            MyTesterImpl servant = new MyTesterImpl();
			
            // Activate servant
            byte[] servantId = myPOA.activate_object(servant);

            // Get servant reference and write it to file
            org.omg.CORBA.Object obj = myPOA.id_to_reference(servantId);
            String reference = orb.object_to_string(obj);
	        
            String dbior = "Tester.ior";
            File ior_fl = new File(dbior);
            try {
                BufferedWriter bw = new BufferedWriter(new FileWriter(ior_fl));
                bw.write(reference, 0, reference.length());
                bw.flush();
                bw.close();
            } catch (java.io.IOException e) {
                System.err.println("[Server] Error writing file " + dbior);
                System.exit(0);
            }

            // Start serving requests
            System.out.println("[Server] Running...");
            orb.run();
			
            //Destroy orb
            orb.destroy();
			
            System.exit(0);

        } catch (Exception e) {
            System.err.println("[Server] Main: Unexpected exception " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
