/*
 * MORFEO Project
 * http://www.morfeo-project.org
 *
 * Component: TIDorbJTest
 * Programming Language: Java
 *
 * File: $Source: EchoClient.java $
 * Version: $Revision: 1.0 $
 * Date: $Date: 10-ago-2006 $
 * Last modified by: $Author: irenka.redondo $
 *
 * (C) Copyright 2004 Telefnica Investigacin y Desarrollo
 *     S.A.Unipersonal (Telefnica I+D)
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 */

import java.util.Properties;
import java.io.*;

import Mod.*;


public class TestClient {
    public static void main(String args[])
    {
    	try {
            // Create and initialize orb
            Properties props = new Properties();
            props.setProperty("org.omg.CORBA.ORBClass", 
                              "es.tid.TIDorbj.core.TIDORB");
            props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                              "es.tid.TIDorbj.core.SingletonORB");
            props.setProperty("es.tid.TIDorbj.trace.level", "4");
            props.setProperty("es.tid.TIDorbj.trace.file", "client.log");
            props.setProperty("es.tid.TIDorbj.bidirectional", "BOTH");

            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,props);

            // Get ior
            String ior_str = null;
            String dbior = "Foo.ior";
            File ior_fl = new File(dbior);
        
            if (ior_fl.exists()) { //ior in file
                try {
                    BufferedReader br = new BufferedReader(new FileReader(ior_fl));
                    ior_str = br.readLine();
                    br.close();
                } catch (java.io.IOException e) {
                    System.err.println("Error reading file " + dbior);
                    System.exit(1);
                }
            }
            else { //ior in standard input
                try {
                    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
                    ior_str = br.readLine();
                    br.close();
                } catch (java.io.IOException e) {
                    System.err.println("Error standard input");
                    System.exit(1);
                }
            }	

            // Get object reference
            org.omg.CORBA.Object obj = orb.string_to_object(ior_str);
            Foo refFoo = FooHelper.narrow(obj);

            String message = "Hola mundo";
            String response = refFoo.echo(message);

            System.out.println("[Client] echo(" + message + ") = " + response);
            
            System.out.println("[Client] Destroying ORB ...");
            orb.destroy();
            
            //System.exit(0);
    	}
    	catch (Exception e) {
            System.err.println("Main: Unexpected exception " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
