/*
 * MORFEO Project
 * http://www.morfeo-project.org
 *
 * Component: TIDorbJTest
 * Programming Language: Java
 *
 * File: $Source: Client.java $
 * Version: $Revision: 1.0 $
 * Date: $Date: 17-ago-2006 $
 * Last modified by: $Author: irenka.redondo $
 *
 * (C) Copyright 2004 Telefnica Investigacin y Desarrollo
 *     S.A.Unipersonal (Telefnica I+D)
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Date;

import org.omg.CORBA.Any;
import org.omg.CORBA.AnyHolder;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.SystemException;

import Marshalling.CalculatedTrackAltitude;
import Marshalling.CalculationMode;
import Marshalling.Hemisphere;
import Marshalling.HorizontalTendency;
import Marshalling.LastMesuredModeC;
import Marshalling.LatLong2DPosition;
import Marshalling.LatLongDirection;
import Marshalling.Latitude;
import Marshalling.Longitude;
import Marshalling.MesuredPosition;
import Marshalling.PlotAge;
import Marshalling.RadarAnyPing;
import Marshalling.RadarAnyPingHelper;
import Marshalling.RadarTrack;
import Marshalling.RadarTrackHelper;
import Marshalling.RadarTracksListHolder;
import Marshalling.RadarUpdate;
import Marshalling.SlantRangeCode;
import Marshalling.SpecialCode;
import Marshalling.SpeedTendency;
import Marshalling.StateVector;
import Marshalling.SurveillanceManagerAsterix30;
import Marshalling.SurveillanceManagerAsterix30Helper;
import Marshalling.TargetType;
import Marshalling.Tendencies;
import Marshalling.TrackStatus;
import Marshalling.TrackType;
import Marshalling.TrackVelocity;
import Marshalling.VerticalTendency;
import Marshalling.eventNum;

public class Client {
	
    private static final int NUMBER_OF_TRIES = 2000;
    
    static long result (Date t1, Date t2) {
        long dif, mili_t1, mili_t2;
        mili_t1 = t1.getTime();
        mili_t2 = t2.getTime();
        dif = mili_t2 - mili_t1;
        return dif*1000; // Resultado en microsegundos
    }
	
    static Object readIOR (String file_name, ORB orb) throws IOException {
        String ior;
        File file = new File(file_name);

        BufferedReader br = new BufferedReader(new FileReader(file));
        ior = br.readLine();
        br.close();
        return orb.string_to_object(ior);
    }

    public static void perform_ATCStructMeasurements (ORB orb, int id) 
        throws IOException {
        
        Object ref = readIOR("SurveillanceManagerAsterix30.ior", orb);
        SurveillanceManagerAsterix30 surveillanceManagerAsterix30Ref = 
            SurveillanceManagerAsterix30Helper.narrow(ref); 

        Date t1, t2;
        long total, tt;

        RadarTrack [] trackEvent = new RadarTrack [eventNum.value];
        for (int j=0; j<trackEvent.length; j++) {
            trackEvent[j] = new RadarTrack();
            trackEvent[j].state_vector = new StateVector();
            trackEvent[j].state_vector.calculated_track_altitude = new CalculatedTrackAltitude();
            trackEvent[j].state_vector.calculated_track_altitude.calculation_mode = 
                CalculationMode.ASSUMED_HEIGHT;
            trackEvent[j].state_vector.last_mesured_mode_C = new LastMesuredModeC();
            trackEvent[j].state_vector.latlong_position = new LatLong2DPosition();
            trackEvent[j].state_vector.latlong_position.latit = new Latitude();
            trackEvent[j].state_vector.latlong_position.latit.lat_hemis = Hemisphere.NORTH;
            trackEvent[j].state_vector.latlong_position.longit = new Longitude();
            trackEvent[j].state_vector.latlong_position.longit.long_dir = LatLongDirection.EAST;
            trackEvent[j].state_vector.Mesured_Position = new MesuredPosition();
            trackEvent[j].state_vector.Plot_Age = new PlotAge();
            trackEvent[j].state_vector.track_status = new TrackStatus();
            trackEvent[j].state_vector.track_status.radar_update = RadarUpdate.PR_MONOTRACK;
            trackEvent[j].state_vector.track_status.slant_range_code = SlantRangeCode.SLR_NOT_CORRECTED;
            trackEvent[j].state_vector.track_status.special_code = SpecialCode.DEFAULT_SPECIAL_CODE;
            trackEvent[j].state_vector.track_status.target_type = TargetType.LIVE_TARGET;
            trackEvent[j].state_vector.track_status.track_type = TrackType.CONFIRMED_TRACK;
            trackEvent[j].state_vector.track_tendencies = new Tendencies();
            trackEvent[j].state_vector.track_tendencies.horizontal = HorizontalTendency.LEFT;
            trackEvent[j].state_vector.track_tendencies.speed = SpeedTendency.ACCELERATE;
            trackEvent[j].state_vector.track_tendencies.vertical = VerticalTendency.CLIMB;
            trackEvent[j].state_vector.track_velocity = new TrackVelocity();
        }
		
        total=0;
        int i;
        for (i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            surveillanceManagerAsterix30Ref.TracksIN(trackEvent);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] SequenceATCStruct IN " + 
                           total/NUMBER_OF_TRIES);
		
        RadarTracksListHolder trackEvent_out = new RadarTracksListHolder();
        total = 0;
        for (i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            surveillanceManagerAsterix30Ref.TracksOUT(trackEvent_out);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] SequenceATCStruct OUT " + 
                           total/NUMBER_OF_TRIES);
		
        //RadarTracksListHolder trackEvent_inOut = new RadarTracksListHolder();
        total = 0;
        for (i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            surveillanceManagerAsterix30Ref.TracksINOUT(trackEvent_out);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] SequenceATCStruct INOUT " + 
                           total/NUMBER_OF_TRIES);

        RadarTrack [] trackEvent_ret = new RadarTrack [eventNum.value];
        total = 0;
        for (i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            trackEvent_ret = surveillanceManagerAsterix30Ref.TracksRET();
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] SequenceATCStruct RETURN " + 
                           total/NUMBER_OF_TRIES);
    }
	
    public static void perform_AnyATCMeasurements (ORB orb, int id, boolean do_shutdown) 
        throws IOException {
        Object ref = readIOR("RadarAnyPing.ior", orb);
        RadarAnyPing anyPingRef = RadarAnyPingHelper.narrow(ref);
		
        Date t1, t2;
        long total, tt;
        int i;

        Any struct_to_send = orb.create_any();
        RadarTrack rt = new RadarTrack();
        rt.state_vector = new StateVector();
        rt.state_vector.calculated_track_altitude = new CalculatedTrackAltitude();
        rt.state_vector.calculated_track_altitude.calculation_mode = 
            CalculationMode.ASSUMED_HEIGHT;
        rt.state_vector.last_mesured_mode_C = new LastMesuredModeC();
        rt.state_vector.latlong_position = new LatLong2DPosition();
        rt.state_vector.latlong_position.latit = new Latitude();
        rt.state_vector.latlong_position.latit.lat_hemis = Hemisphere.NORTH;
        rt.state_vector.latlong_position.longit = new Longitude();
        rt.state_vector.latlong_position.longit.long_dir = LatLongDirection.EAST;
        rt.state_vector.Mesured_Position = new MesuredPosition();
        rt.state_vector.Plot_Age = new PlotAge();
        rt.state_vector.track_status = new TrackStatus();
        rt.state_vector.track_status.radar_update = RadarUpdate.PR_MONOTRACK;
        rt.state_vector.track_status.slant_range_code = SlantRangeCode.SLR_NOT_CORRECTED;
        rt.state_vector.track_status.special_code = SpecialCode.DEFAULT_SPECIAL_CODE;
        rt.state_vector.track_status.target_type = TargetType.LIVE_TARGET;
        rt.state_vector.track_status.track_type = TrackType.CONFIRMED_TRACK;
        rt.state_vector.track_tendencies = new Tendencies();
        rt.state_vector.track_tendencies.horizontal = HorizontalTendency.LEFT;
        rt.state_vector.track_tendencies.speed = SpeedTendency.ACCELERATE;
        rt.state_vector.track_tendencies.vertical = VerticalTendency.CLIMB;
        rt.state_vector.track_velocity = new TrackVelocity();
        RadarTrackHelper.insert(struct_to_send, rt);
        total = 0;
        for (i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            anyPingRef.pongIn(struct_to_send);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] AnyATCStruct IN " + 
                           total/NUMBER_OF_TRIES);
		
        AnyHolder struct_to_out = new AnyHolder();
        total = 0;
        for (i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            anyPingRef.pongOut(struct_to_out);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] AnyATCStruct OUT " + 
                           total/NUMBER_OF_TRIES);
		
        AnyHolder struct_to_inOut = new AnyHolder();
        struct_to_inOut.value = orb.create_any();
        RadarTrackHelper.insert(struct_to_inOut.value, rt);
        total = 0;
        for (i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            anyPingRef.pongInOut(struct_to_inOut);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] AnyATCStruct INOUT " + 
                           total/NUMBER_OF_TRIES);

        Any struct_to_ret;
        total = 0;
        for (i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            struct_to_ret = anyPingRef.pongRet();
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] AnyATCStruct RETURN " + 
                           total/NUMBER_OF_TRIES);

        // Take care with the following if lunch several clients!!
        // if (do_shutdown) {
        //      anyPingRef.cerrar();
        //  }

    }

    public static void main (String [] args) {
        try {
            if (args.length < 1) {
                System.err.println("Usage: Client client_id");
                System.exit(1);
            }
            int clid = (new Integer(args[0])).intValue();

            // Create and initialize orb
            /*
            Properties props = new Properties();
            props.setProperty("org.omg.CORBA.ORBClass", 
                              "es.tid.TIDorbj.core.TIDORB");
            props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                              "es.tid.TIDorbj.core.SingletonORB");
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,props);
            */
			
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,System.getProperties());
    		
            try {    
                perform_ATCStructMeasurements(orb, clid);
            } catch (org.omg.CORBA.NO_RESPONSE e) {
                System.err.println("[Client " + clid + "] " + 
                                   "CORBA NO_RESPONSE exception received");
            }
            try {    
                perform_AnyATCMeasurements(orb, clid, false);
            } catch (org.omg.CORBA.NO_RESPONSE e) {
                System.err.println("[Client " + clid + "] " + 
                                   "CORBA NO_RESPONSE exception received");
            }


            Thread.sleep(10000);

            try {    
                perform_ATCStructMeasurements(orb, clid);
            } catch (org.omg.CORBA.NO_RESPONSE e) {
                System.err.println("[Client " + clid + "] " + 
                                   "CORBA NO_RESPONSE exception received");
            }
            try {    
                perform_AnyATCMeasurements(orb, clid, true);
            } catch (org.omg.CORBA.NO_RESPONSE e) {
                System.err.println("[Client " + clid + "] " + 
                                   "CORBA NO_RESPONSE exception received");
            }

            
            //Thread.sleep(10000);

            System.exit(0);

        } catch (SystemException e) {
            System.err.println("Main: org.omg.CORBA.SystemException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (IOException e) {
            System.err.println("Main: java.io.IOException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (Exception e) {
            System.err.println("Main: Unexpected exception " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
