/*
 * MORFEO Project
 * http://www.morfeo-project.org
 *
 * Component: TIDorbJTest
 * Programming Language: Java
 *
 * File: $Source: RadarAnyPngImpl.java $
 * Version: $Revision: 1.0 $
 * Date: $Date: 17-ago-2006 $
 * Last modified by: $Author: irenka.redondo $
 *
 * (C) Copyright 2004 Telefnica Investigacin y Desarrollo
 *     S.A.Unipersonal (Telefnica I+D)
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 */

import org.omg.CORBA.Any;
import org.omg.CORBA.AnyHolder;
import org.omg.CORBA.ORB;

import Marshalling.CalculatedTrackAltitude;
import Marshalling.CalculationMode;
import Marshalling.Hemisphere;
import Marshalling.HorizontalTendency;
import Marshalling.LastMesuredModeC;
import Marshalling.LatLong2DPosition;
import Marshalling.LatLongDirection;
import Marshalling.Latitude;
import Marshalling.Longitude;
import Marshalling.MesuredPosition;
import Marshalling.PlotAge;
import Marshalling.RadarAnyPingPOA;
import Marshalling.RadarTrack;
import Marshalling.RadarTrackHelper;
import Marshalling.RadarUpdate;
import Marshalling.SlantRangeCode;
import Marshalling.SpecialCode;
import Marshalling.SpeedTendency;
import Marshalling.StateVector;
import Marshalling.TargetType;
import Marshalling.Tendencies;
import Marshalling.TrackStatus;
import Marshalling.TrackType;
import Marshalling.TrackVelocity;
import Marshalling.VerticalTendency;

public class RadarAnyPingImpl extends RadarAnyPingPOA {

    private org.omg.CORBA.ORB m_orb;

    public RadarAnyPingImpl (org.omg.CORBA.ORB orb) {
        this.m_orb = orb;
    }

    public void pongIn(Any xArg) {
        // TODO Auto-generated method stub
        
    }
    
    public void pongOut(AnyHolder xArg) {
        RadarTrack rt = new RadarTrack();
        rt.state_vector = new StateVector();
        rt.state_vector.calculated_track_altitude = new CalculatedTrackAltitude();
        rt.state_vector.calculated_track_altitude.calculation_mode = 
            CalculationMode.ASSUMED_HEIGHT;
        rt.state_vector.last_mesured_mode_C = new LastMesuredModeC();
        rt.state_vector.latlong_position = new LatLong2DPosition();
        rt.state_vector.latlong_position.latit = new Latitude();
        rt.state_vector.latlong_position.latit.lat_hemis = Hemisphere.NORTH;
        rt.state_vector.latlong_position.longit = new Longitude();
        rt.state_vector.latlong_position.longit.long_dir = LatLongDirection.EAST;
        rt.state_vector.Mesured_Position = new MesuredPosition();
        rt.state_vector.Plot_Age = new PlotAge();
        rt.state_vector.track_status = new TrackStatus();
        rt.state_vector.track_status.radar_update = RadarUpdate.PR_MONOTRACK;
        rt.state_vector.track_status.slant_range_code = SlantRangeCode.SLR_NOT_CORRECTED;
        rt.state_vector.track_status.special_code = SpecialCode.DEFAULT_SPECIAL_CODE;
        rt.state_vector.track_status.target_type = TargetType.LIVE_TARGET;
        rt.state_vector.track_status.track_type = TrackType.CONFIRMED_TRACK;
        rt.state_vector.track_tendencies = new Tendencies();
        rt.state_vector.track_tendencies.horizontal = HorizontalTendency.LEFT;
        rt.state_vector.track_tendencies.speed = SpeedTendency.ACCELERATE;
        rt.state_vector.track_tendencies.vertical = VerticalTendency.CLIMB;
        rt.state_vector.track_velocity = new TrackVelocity();

        //xArg.value = ORB.init().create_any();
        xArg.value = m_orb.create_any();
        RadarTrackHelper.insert(xArg.value, rt);
    }

    public void pongInOut(AnyHolder xArg) {
        // TODO Auto-generated method stub

    }

    public Any pongRet() {
        //Any any_to_ret = ORB.init().create_any();
        Any any_to_ret = m_orb.create_any();

        RadarTrack rt = new RadarTrack();
        rt.state_vector = new StateVector();
        rt.state_vector.calculated_track_altitude = new CalculatedTrackAltitude();
        rt.state_vector.calculated_track_altitude.calculation_mode = 
            CalculationMode.ASSUMED_HEIGHT;
        rt.state_vector.last_mesured_mode_C = new LastMesuredModeC();
        rt.state_vector.latlong_position = new LatLong2DPosition();
        rt.state_vector.latlong_position.latit = new Latitude();
        rt.state_vector.latlong_position.latit.lat_hemis = Hemisphere.NORTH;
        rt.state_vector.latlong_position.longit = new Longitude();
        rt.state_vector.latlong_position.longit.long_dir = LatLongDirection.EAST;
        rt.state_vector.Mesured_Position = new MesuredPosition();
        rt.state_vector.Plot_Age = new PlotAge();
        rt.state_vector.track_status = new TrackStatus();
        rt.state_vector.track_status.radar_update = RadarUpdate.PR_MONOTRACK;
        rt.state_vector.track_status.slant_range_code = SlantRangeCode.SLR_NOT_CORRECTED;
        rt.state_vector.track_status.special_code = SpecialCode.DEFAULT_SPECIAL_CODE;
        rt.state_vector.track_status.target_type = TargetType.LIVE_TARGET;
        rt.state_vector.track_status.track_type = TrackType.CONFIRMED_TRACK;
        rt.state_vector.track_tendencies = new Tendencies();
        rt.state_vector.track_tendencies.horizontal = HorizontalTendency.LEFT;
        rt.state_vector.track_tendencies.speed = SpeedTendency.ACCELERATE;
        rt.state_vector.track_tendencies.vertical = VerticalTendency.CLIMB;
        rt.state_vector.track_velocity = new TrackVelocity();
		
        RadarTrackHelper.insert(any_to_ret, rt);
        return any_to_ret;
    }

    public void cerrar() {
        if (m_orb != null)	
            m_orb.shutdown(false);
    }

}
