/*
 * MORFEO Project
 * http://www.morfeo-project.org
 *
 * Component: TIDorbJTest
 * Programming Language: Java
 *
 * File: $Source: Client.java $
 * Version: $Revision: 1.0 $
 * Date: $Date: 17-ago-2006 $
 * Last modified by: $Author: irenka.redondo $
 *
 * (C) Copyright 2004 Telefnica Investigacin y Desarrollo
 *     S.A.Unipersonal (Telefnica I+D)
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 */
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Date;

import org.omg.CORBA.CharHolder;
import org.omg.CORBA.FloatHolder;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.StringHolder;
import org.omg.CORBA.SystemException;

import Marshalling.CharPing;
import Marshalling.CharPingHelper;
import Marshalling.DummyPing;
import Marshalling.DummyPingHelper;
import Marshalling.FloatPing;
import Marshalling.FloatPingHelper;
import Marshalling.StringPing;
import Marshalling.StringPingHelper;

public class Client {
	
    private static final int NUMBER_OF_TRIES = 10000;
	
    static long result (Date t1, Date t2) {
        long dif, mili_t1, mili_t2;
        mili_t1 = t1.getTime();
        mili_t2 = t2.getTime();
        dif = mili_t2 - mili_t1;
        return dif*1000; //resultado en microsegundos
    }
	
    static Object readIOR (String file_name, ORB orb) throws IOException {
        String ior;
        File file = new File(file_name);

        BufferedReader br = new BufferedReader(new FileReader(file));
        ior = br.readLine();
        br.close();
        return orb.string_to_object(ior);
    }

    public static void perform_sendMeasurements (ORB orb, int id) throws IOException {
        Object ref = readIOR("DummyPing.ior", orb);
        DummyPing charPingRef = DummyPingHelper.narrow(ref);
        Date t1, t2;
        long total = 0, tt;

        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            charPingRef.dummyPong();
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] PING " + total/NUMBER_OF_TRIES);
		
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            charPingRef.dummyPongOneWay();
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] PING ONEWAY " + total/NUMBER_OF_TRIES);
		
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
	
    public static void perform_charMeasurements (char char_to_send, ORB orb, int id) 
        throws IOException {
        Object ref = readIOR("CharPing.ior", orb);
        CharPing charPingRef = CharPingHelper.narrow(ref);
		
        Date t1, t2;
        long total = 0, tt;

        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            charPingRef.pongIn(char_to_send);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] Char IN " + total/NUMBER_OF_TRIES);
		
        CharHolder char_to_out = new CharHolder();
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            charPingRef.pongOut(char_to_out);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] Char OUT " + total/NUMBER_OF_TRIES);
		
        CharHolder char_to_inOut = new CharHolder();
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            charPingRef.pongInOut(char_to_inOut);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] Char INOUT " + total/NUMBER_OF_TRIES);

        char char_to_ret;
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            char_to_ret = charPingRef.pongRet();
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] Char RETURN " + total/NUMBER_OF_TRIES);
    }

    public static void perform_floatMeasurements (ORB orb, int id) throws IOException {
        Object ref = readIOR("FloatPing.ior", orb);
        FloatPing floatPingRef = FloatPingHelper.narrow(ref);
		
        Date t1, t2;
        long total = 0, tt;

        float float_to_in = 1;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            floatPingRef.pongIn(float_to_in);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] Float IN " + total/NUMBER_OF_TRIES);
		
        FloatHolder float_to_out = new FloatHolder(1);
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            floatPingRef.pongOut(float_to_out);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] Float OUT " + total/NUMBER_OF_TRIES);
		
        FloatHolder float_to_inOut = new FloatHolder(1);
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            floatPingRef.pongInOut(float_to_inOut);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] Float INOUT " + total/NUMBER_OF_TRIES);

        float float_to_ret;
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            float_to_ret = floatPingRef.pongRet();
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] Float RETURN " + total/NUMBER_OF_TRIES);
    }

    public static void perform_stringMeasurements (ORB orb, int id) throws IOException {
        Object ref = readIOR("StringPing.ior", orb);
        StringPing stringPingRef = StringPingHelper.narrow(ref);
		
        Date t1, t2;
        long total = 0, tt;

        String string_to_send = "str1";
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            stringPingRef.pongIn(string_to_send);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] String IN " + total/NUMBER_OF_TRIES);
		
        StringHolder string_to_out = new StringHolder("hello");
        //StringHolder string_to_out = new StringHolder();
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            stringPingRef.pongOut(string_to_out);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] String OUT " + total/NUMBER_OF_TRIES);
		
        StringHolder string_to_inOut = new StringHolder("holitas");
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            stringPingRef.pongInOut(string_to_inOut);
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] String INOUT " + total/NUMBER_OF_TRIES);

        String string_to_ret;
        total = 0;
        for (int i=0; i<NUMBER_OF_TRIES; i++) {
            t1 = new Date();
            string_to_ret = stringPingRef.pongRet();
            t2 = new Date();
            tt = result(t1, t2);
            total = total + tt;
        }
        System.out.println("[Client " + id + "] String RETURN " + total/NUMBER_OF_TRIES);
    }

    public static void main (String [] args) {
        try {
            if (args.length < 1) {
                System.err.println("Usage: Client client_id");
                System.exit(1);
            }
            int clid = (new Integer(args[0])).intValue();
				
			
            // Create and initialize orb
            /*
            Properties props = new Properties();
            props.setProperty("org.omg.CORBA.ORBClass", 
                              "es.tid.TIDorbj.core.TIDORB");
            props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                              "es.tid.TIDorbj.core.SingletonORB");

            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,props);
            */
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,System.getProperties());


            System.out.println("[Client] Starting...");

            try {    
                perform_sendMeasurements(orb, clid);
            } catch (org.omg.CORBA.NO_RESPONSE e) {
                System.err.println("[Client " + clid + "] " + 
                                   "CORBA NO_RESPONSE exception received");
            }
            try {    
                perform_charMeasurements('a', orb, clid);
            } catch (org.omg.CORBA.NO_RESPONSE e) {
                System.err.println("[Client " + clid + "] " + 
                                   "CORBA NO_RESPONSE exception received");
            }
            try {    
                perform_floatMeasurements(orb, clid);
            } catch (org.omg.CORBA.NO_RESPONSE e) {
                System.err.println("[Client " + clid + "] " + 
                                   "CORBA NO_RESPONSE exception received");
            }
            try {    
                perform_stringMeasurements(orb, clid);
            } catch (org.omg.CORBA.NO_RESPONSE e) {
                System.err.println("[Client " + clid + "] " + 
                                   "CORBA NO_RESPONSE exception received");
            }
    		
            System.exit(0);
    		
        } catch (SystemException e) {
            System.err.println("Main: org.omg.CORBA.SystemException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (IOException e) {
            System.err.println("Main: java.io.IOException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (Exception e) {
            System.err.println("Main: Unexpected exception " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}

