/*
 * MORFEO Project
 * http://www.morfeo-project.org
 *
 * Component: TIDorbJTest
 * Programming Language: Java
 *
 * File: $Source: Server.java $
 * Version: $Revision: 1.0 $
 * Date: $Date: 17-ago-2006 $
 * Last modified by: $Author: irenka.redondo $
 *
 * (C) Copyright 2004 Telefnica Investigacin y Desarrollo
 *     S.A.Unipersonal (Telefnica I+D)
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 */
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.UserException;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

public class Server {
    public static ORB orb;
    
    static void writeIOR2File (String file_name, ORB orb, Object ref) throws IOException {
        String ior = orb.object_to_string(ref);
        File file = new File(file_name);
        
        BufferedWriter bw = new BufferedWriter(new FileWriter(file));
        bw.write(ior, 0, ior.length());
        bw.flush();
        bw.close();
    }
	
    public static void main(String[] args) {
        try {
            // Create and initialize orb

            /*
            Properties props = new Properties();
            props.setProperty("org.omg.CORBA.ORBClass", 
                              "es.tid.TIDorbj.core.TIDORB");
            props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                              "es.tid.TIDorbj.core.SingletonORB");
                      
            ORB orb = ORB.init(args, props);
            */
            ORB orb = ORB.init(args, System.getProperties());
                    
            Object poa_ref = orb.resolve_initial_references("RootPOA");
            POA root_poa = POAHelper.narrow(poa_ref);
			
            DummyPingImpl servantDummy = new DummyPingImpl();
            Object dummy_ref = root_poa.servant_to_reference(servantDummy);
            writeIOR2File("DummyPing.ior", orb, dummy_ref);
			
            CharPingImpl servantChar = new CharPingImpl();
            Object char_ref = root_poa.servant_to_reference(servantChar);
            writeIOR2File("CharPing.ior", orb, char_ref);
			
            FloatPingImpl servantFloat = new FloatPingImpl();
            Object float_ref = root_poa.servant_to_reference(servantFloat);
            writeIOR2File("FloatPing.ior", orb, float_ref);
			
            StringPingImpl servantString = new StringPingImpl();
            Object string_ref = root_poa.servant_to_reference(servantString);
            writeIOR2File("StringPing.ior", orb, string_ref);
			
            root_poa.the_POAManager().activate();
            System.out.println("[Server] Running...");
            orb.run();
			
            orb.destroy();
			
            System.exit(0);
	
        } catch (SystemException e) {
            System.err.println("Main: org.omg.CORBA.SystemException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (UserException e) {
            System.err.println("Main: org.omg.CORBA.UserException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (IOException e) {
            System.err.println("Main: java.io.IOException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (Exception e) {
            System.err.println("Main: Unexpected exception " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}

