/*
* MORFEO Project
* http://www.morfeo-project.org
*
* Component: TIDorbJTest
* Programming Language: Java
*
* File: $Source: Client.java $
* Version: $Revision: 1.0 $
* Date: $Date: 22-sep-2006 $
* Last modified by: $Author: irenka.redondo $
*
* (C) Copyright 2004 Telefnica Investigacin y Desarrollo
*     S.A.Unipersonal (Telefnica I+D)
*
* Info about members and contributors of the MORFEO project
* is available at:
*
*   http://www.morfeo-project.org/TIDorbJ/CREDITS
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*
* If you want to use this software an plan to distribute a
* proprietary application in any way, and you are not licensing and
* distributing your source code under GPL, you probably need to
* purchase a commercial license of the product.  More info about
* licensing options is available at:
*
*   http://www.morfeo-project.org/TIDorbJ/Licensing
*/

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Date;

import org.omg.CORBA.Any;
import org.omg.CORBA.AnyHolder;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.SystemException;

import Test.ComplexData;
import Test.ComplexDataHelper;
import Test.Person;
import Test.Place;
import Test.Point;
import Test.Tester1;
import Test.Tester1Helper;
import Test.Tester6;
import Test.Tester6Helper;


import Marshalling.*;

public class Client {
	
    private static final int OBJREF_BUFFER_SIZE = 1000;

    void terminarEjecucion (int a) {
        //this.foo_ref.shutdown();
        System.exit(0);
    }
    
    static void buildString (int n, String c) {
        c = new String();
        for (int i=0; i<n; i++)
            c = c + 'a';
    }
	
    public static long timeDiff (Date t1, Date t2) {
        long diff, mili_t1, mili_t2;
        mili_t1 = t1.getTime();
        mili_t2 = t2.getTime();
        diff = mili_t2 - mili_t1;
        return diff*1000; // Resultado en microsegundos
    }
	
    static Object readIOR (String file_name, ORB orb) throws IOException {
        String ior;
        File file = new File(file_name);

        BufferedReader br = new BufferedReader(new FileReader(file));
        ior = br.readLine();
        br.close();
        return orb.string_to_object(ior);
    }
	
    public static void main (String [] args) {
        // Argumentos de la lnea de comandos
        if (args.length < 3) {
            System.err.println("Usage: Client client_id events_num phase_num");
            System.exit(1);
        }
        int clid = (new Integer(args[0])).intValue();
        int nevents = (new Integer(args[1])).intValue();
        int phase = (new Integer(args[2])).intValue();
			
        // Crea el fichero donde se almacenarn los resultados
        /*
        String name_result_file = "client_" + clid + ".result";
        if (System.getProperty("org.omg.CORBA.ORBClass").equals("es.tid.TIDorbj.core.TIDORB")) 
            name_result_file = name_result_file + "_TIDorbJ";
        else if (System.getProperty("org.omg.CORBA.ORBClass").equals("org.jacorb.orb.ORB"));
        name_result_file = name_result_file + "_JacORB";
        
        File file = new File(name_result_file);
        */
			
        try {
            //	BufferedWriter bw = new BufferedWriter(new FileWriter(file));
            
            // Create and initialize orb
            /*
            Properties props = new Properties();
            props.setProperty("org.omg.CORBA.ORBClass", 
                              "es.tid.TIDorbj.core.TIDORB");
            props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                              "es.tid.TIDorbj.core.SingletonORB");
            
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,props);
            */
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, System.getProperties());
    		
            // Not supported by Visibroker
            /*
              String pwd, uri;
              pwd = System.getProperty("user.dir");
              uri = "file://" + pwd + "/tester1.ior";
   	    	
              Object obj5 = orb.string_to_object(uri);
            */
            

            //
            // Read reference to tester 1
            //
            Object obj1 = readIOR("Tester1.ior", orb);
            Tester1 tester_ref = Tester1Helper.narrow(obj1);
   	    	
            int tamano = 1096;
            String cadena = new String();
            buildString(tamano, cadena);
   	    	
            Date t1, t2;
            long total=0, tt;


            //
            // Creates and initialices datas
            //

            // Simple data
            Any any_simple = orb.create_any();
            any_simple.insert_string(cadena);
   	    	
            // Complex data
            Any any_complex = orb.create_any();
            ComplexData data = new ComplexData();
            data.buffer = new byte[OBJREF_BUFFER_SIZE];
            data.when = new Test.Date();
            data.when.hour = 456;
            data.name = cadena;
            data.employes = new Person[30];
            for (int i=0; i<30; i++) {
                String cadena2 = new String();
                buildString(3+(70*i), cadena2);
                data.employes[i] = new Person();
                data.employes[i].name = "Nombre1";
                data.employes[i].last_name = "Nombre2";
                data.employes[i].alias = cadena2;
                data.employes[i].age = (new Integer((i*2) + 5)).shortValue();
                data.employes[i].from = new Place();
                data.employes[i].from.p = new Point();
                data.employes[i].from.p.x = (new Integer(3+i)).shortValue();
                data.employes[i].from.p.y = (new Integer(4+i)).shortValue();
                data.employes[i].from.p.z = (new Integer(9+i)).shortValue();
                data.employes[i].from.name = "Nombre1";
            }
            data.places = new Place[10];
            for (int i=0; i<10; i++) {
                data.places[i] = new Place();
                data.places[i].name = "Nombre1";
                data.places[i].p = new Point();
            }
            ComplexDataHelper.insert(any_complex, data);

            // RadarTrack data
            RadarTrack [] trackEvent = new RadarTrack [eventNum.value];
            for (int j=0; j<trackEvent.length; j++) {
                trackEvent[j] = 
                    new RadarTrack();
                trackEvent[j].state_vector = 
                    new StateVector();
                trackEvent[j].state_vector.calculated_track_altitude = 
                    new CalculatedTrackAltitude();
                trackEvent[j].state_vector.calculated_track_altitude.calculation_mode = 
                    CalculationMode.ASSUMED_HEIGHT;
                trackEvent[j].state_vector.last_mesured_mode_C = 
                    new LastMesuredModeC();
                trackEvent[j].state_vector.latlong_position = 
                    new LatLong2DPosition();
                trackEvent[j].state_vector.latlong_position.latit = 
                    new Latitude();
                trackEvent[j].state_vector.latlong_position.latit.lat_hemis = 
                    Hemisphere.NORTH;
                trackEvent[j].state_vector.latlong_position.longit = 
                    new Longitude();
                trackEvent[j].state_vector.latlong_position.longit.long_dir = 
                    LatLongDirection.EAST;
                trackEvent[j].state_vector.Mesured_Position = 
                    new MesuredPosition();
                trackEvent[j].state_vector.Plot_Age = 
                    new PlotAge();
                trackEvent[j].state_vector.track_status = 
                    new TrackStatus();
                trackEvent[j].state_vector.track_status.radar_update = 
                    RadarUpdate.PR_MONOTRACK;
                trackEvent[j].state_vector.track_status.slant_range_code = 
                    SlantRangeCode.SLR_NOT_CORRECTED;
                trackEvent[j].state_vector.track_status.special_code = 
                    SpecialCode.DEFAULT_SPECIAL_CODE;
                trackEvent[j].state_vector.track_status.target_type = 
                    TargetType.LIVE_TARGET;
                trackEvent[j].state_vector.track_status.track_type = 
                    TrackType.CONFIRMED_TRACK;
                trackEvent[j].state_vector.track_tendencies = 
                    new Tendencies();
                trackEvent[j].state_vector.track_tendencies.horizontal = 
                    HorizontalTendency.LEFT;
                trackEvent[j].state_vector.track_tendencies.speed = 
                    SpeedTendency.ACCELERATE;
                trackEvent[j].state_vector.track_tendencies.vertical = 
                    VerticalTendency.CLIMB;
                trackEvent[j].state_vector.track_velocity = 
                    new TrackVelocity();
            }
            

   	    	
            try {
                switch(phase) {
                case 1: {
                    //
                    // Phase 1
                    //
                    System.out.println("[Phase 1]");
                    int i;
                    tester_ref.register_client(clid, nevents);
						
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        tester_ref.push1_phase1(any_simple);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* String res = "[Phase1] Push1 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push1 Simple Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        tester_ref.push1_phase1(any_complex);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* res = "[Phase1] Push1 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push1 Complex Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        Any my_any = tester_ref.push2_phase1(any_simple);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* res = "[Phase1] Push2 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push2 Simple Any " + 
                                       total/nevents);
						
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        Any my_any = tester_ref.push2_phase1(any_complex);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* res = "[Phase1] Push2 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push2 Complex Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        tester_ref.push3_phase1(any_simple);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* res = "[Phase1] Push3 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push3 Simple Any " + 
                                       total/nevents);
						
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        tester_ref.push3_phase1(any_complex);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* res = "[Phase1] Push3 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push3 Complex Any " + 
                                       total/nevents);
                } break;
						
                case 2: {
                    //
                    // Phase 2
                    //
                    System.out.println("[Phase 2]");
                    int i;
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push1_phase2(any_simple);
                    }
                    /* res = "[Phase2] Push1 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length());  */
                    System.out.println("[Client " + clid + "] Push1 Simple Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push1_phase2(any_complex);
                    }
                    /* res = "[Phase2] Push1 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push1 Complex Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push2_phase2(any_simple);
                    }
                    /* res = "[Phase2] Push2 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push2 Simple Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push2_phase2(any_complex);
                    }
                    /* res = "[Phase2] Push2 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push2 Complex Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push3_phase2(any_simple);
                    }
                    /* res = "[Phase2] Push3 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push3 Simple Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push3_phase2(any_complex);
                    }
                    /* res = "[Phase2] Push3 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push3 Complex Any " + 
                                       total/nevents);

                } break;
						
                case 3: {
                    //
                    // Phase 3
                    //
                    System.out.println("[Phase 3]");
                    int i;
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push1_phase3(any_simple);
                    }
                    /* res = "[Phase3] Push1 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push1 Simple Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push1_phase3(any_complex);
                    }
                    /* res = "[Phase3] Push1 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push1 Complex Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push2_phase3(any_simple);
                    }
                    /* res = "[Phase3] Push2 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push2 Simple Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push2_phase3(any_complex);
                    }
                    /* res = "[Phase3] Push2 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push2 Complex Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push3_phase3(any_simple);
                    }
                    /* res = "[Phase3] Push3 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push3 Simple Any " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        total = total + tester_ref.push3_phase3(any_complex);
                    }
                    /* res = "[Phase3] Push3 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] Push3 Complex Any " + 
                                       total/nevents);

                } break;
						
                case 4: {
                    //
                    // Phase 4
                    // 
                    System.out.println("[Phase 4]");

                    Object obj4 = readIOR("SurveillanceManagerAsterix30.ior", orb);
                    SurveillanceManagerAsterix30 surveillance_ref = 
                        SurveillanceManagerAsterix30Helper.narrow(obj4); 

                    //surveillance_ref.register_client(clid, nevents);

                    total=0;
                    int i;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        surveillance_ref.TracksIN(trackEvent);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    System.out.println("[Client " + clid + "] SequenceATCStruct IN " + 
                                       total/nevents);
                    
                    RadarTracksListHolder trackEvent_out = new RadarTracksListHolder();
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        surveillance_ref.TracksOUT(trackEvent_out);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    System.out.println("[Client " + clid + "] SequenceATCStruct OUT " + 
                                       total/nevents);
                    
                    //RadarTracksListHolder trackEvent_inOut = new RadarTracksListHolder();
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        surveillance_ref.TracksINOUT(trackEvent_out);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    System.out.println("[Client " + clid + "] SequenceATCStruct INOUT " + 
                                       total/nevents);

                    RadarTrack [] trackEvent_ret = new RadarTrack [eventNum.value];
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        trackEvent_ret = surveillance_ref.TracksRET();
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    System.out.println("[Client " + clid + "] SequenceATCStruct RETURN " + 
                                       total/nevents);
                } break;

                case 5: {
                    //
                    // Phase 5
                    // 
                    System.out.println("[Phase 5]");
                    
                    Object obj5 = readIOR("RadarAnyPing.ior", orb);
                    RadarAnyPing anyPingRef = RadarAnyPingHelper.narrow(obj5);

                    Any struct_to_send = orb.create_any();
                    RadarTrackHelper.insert(struct_to_send, trackEvent[0]);
                    int i;
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        anyPingRef.pongIn(struct_to_send);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    System.out.println("[Client " + clid + "] AnyATCStruct IN " + 
                                       total/nevents);
                    
                    AnyHolder struct_to_out = new AnyHolder();
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        anyPingRef.pongOut(struct_to_out);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    System.out.println("[Client " + clid + "] AnyATCStruct OUT " + 
                                       total/nevents);
                    
                    AnyHolder struct_to_inOut = new AnyHolder();
                    struct_to_inOut.value = orb.create_any();
                    RadarTrackHelper.insert(struct_to_inOut.value, trackEvent[0]);
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        anyPingRef.pongInOut(struct_to_inOut);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    System.out.println("[Client " + clid + "] AnyATCStruct INOUT " + 
                                       total/nevents);
                    
                    Any struct_to_ret;
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        struct_to_ret = anyPingRef.pongRet();
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    System.out.println("[Client " + clid + "] AnyATCStruct RETURN " + 
                                       total/nevents);
                    

                } break;

                case 6: {
                    //
                    // Phase 6
                    // 
                    System.out.println("[Phase 6]");
                    
                    //
                    // Read reference to tester 6
                    //
                    Object obj6 = readIOR("Tester6.ior", orb);
                    Tester6 tester6_ref = Tester6Helper.narrow(obj6);

                    byte[] bytes_to_in = new byte[1000];
                    byte[] value = new byte[1];
                    byte[] bytes_to_ret = null;
                    Test.ByteSequenceHolder bytes_to_out = new Test.ByteSequenceHolder(value);

                    int i = 0;
                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        tester6_ref.opIN(bytes_to_in);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* String res = "[Phase1] Push1 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] opIN " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        tester6_ref.opOUT(bytes_to_out);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* res = "[Phase1] Push1 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] opOUT " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        tester6_ref.opINOUT(bytes_to_out);
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* res = "[Phase1] Push2 Simple Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] opINOUT " + 
                                       total/nevents);

                    total = 0;
                    for (i=0; i<nevents; i++) {
                        t1 = new Date();
                        bytes_to_ret = tester6_ref.opRET();
                        t2 = new Date();
                        tt = timeDiff(t1, t2);
                        total = total + tt;
                    }
                    /* res = "[Phase1] Push2 Complex Any " + total/nevents + "\n";
                       bw.write(res, 0, res.length()); */
                    System.out.println("[Client " + clid + "] opRET " + 
                                       total/nevents);

                    } break;



                }
            } catch (SystemException e) {
                System.err.println("Main: org.omg.CORBA.SystemException at phase " + 
                                   phase + ": " + e.getMessage());
                e.printStackTrace();
                System.exit(1);
                /*
            } catch (IOException e) {
                System.err.println("Main: java.io.IOException at phase " + phase + ": " + 
                                   e.getMessage());
                e.printStackTrace();
                System.exit(1);
                */
            } catch (Exception e) {
                System.err.println("Main: Unexpected exception at phase " + phase + ": " + 
                                   e.getMessage());
                e.printStackTrace();
                System.exit(1);
            }			
            // cierra el fichero de los resultados
            /*
              bw.flush();
              bw.close();
            */

            // System.out.println("[Client " + clid + "] Finished");
            System.exit(0);
			
        } catch (SystemException e) {
            System.err.println("Main: org.omg.CORBA.SystemException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
            /*		
        } catch (IOException e) {
            System.err.println("Main: java.io.IOException " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
            */
        } catch (Exception e) {
            System.err.println("Main: Unexpected exception " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
			
    }
}
