/*
* MORFEO Project
* http://www.morfeo-project.org
*
* Component: TIDorbJTest
* Programming Language: Java
*
* File: $Source: ClientMT.java $
* Version: $Revision: 1.0 $
* Date: $Date: 22-sep-2006 $
* Last modified by: $Author: irenka.redondo $
*
* (C) Copyright 2004 Telefnica Investigacin y Desarrollo
*     S.A.Unipersonal (Telefnica I+D)
*
* Info about members and contributors of the MORFEO project
* is available at:
*
*   http://www.morfeo-project.org/TIDorbJ/CREDITS
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*
* If you want to use this software an plan to distribute a
* proprietary application in any way, and you are not licensing and
* distributing your source code under GPL, you probably need to
* purchase a commercial license of the product.  More info about
* licensing options is available at:
*
*   http://www.morfeo-project.org/TIDorbJ/Licensing
*/

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Date;

import org.omg.CORBA.Any;
import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.SystemException;

import Test.ComplexData;
import Test.ComplexDataHelper;
import Test.Person;
import Test.Place;
import Test.Point;
import Test.Tester1;
import Test.Tester1Helper;

public class ClientMT {
	
	static Object readIOR (String file_name, ORB orb) throws IOException {
		String ior;
		File file = new File(file_name);

		BufferedReader br = new BufferedReader(new FileReader(file));
		ior = br.readLine();
		br.close();
		return orb.string_to_object(ior);
	}
	
	public static void main (String [] args) {
		// Argumentos de la lnea de comandos
		if (args.length < 4) {
			System.err.println("Usage: ClientMT client_id events_num phase_num threads_num");
			System.exit(1);
		}
		int clid = (new Integer(args[0])).intValue();
		int nevents = (new Integer(args[1])).intValue();
		int phase = (new Integer(args[2])).intValue();
		int nthreads = (new Integer(args[3])).intValue();
		
		try {
			// Inicializa el orb
    		/*Properties props = new Properties();
    		props.setProperty("org.omg.CORBA.ORBClass", "es.tid.TIDorbj.core.TIDORB");
    		props.setProperty("org.omg.CORBA.ORBSingletonClass", "es.tid.TIDorbj.core.SingletonORB");

    		org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,props);*/
			org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args,System.getProperties());
    		
    		// Lee de fichero la referencia al servidor
			
			//Not supported by Visibroker
   	    	/*String pwd, uri;
   	    	pwd = System.getProperty("user.dir");
   	    	uri = "file://" + pwd + "/tester1.ior";
   	    	
   	    	Object obj5 = orb.string_to_object(uri);*/
			Object obj5 = readIOR("tester1.ior", orb);
   	    	Tester1 tester_ref = Tester1Helper.narrow(obj5);
   	    	
   	    	ClientMT_Thread [] threads_list = new ClientMT_Thread [nthreads]; 
   	    	for (int i=0; i<nthreads; i++) {
   	    		threads_list[i] = new ClientMT_Thread(orb, clid, nevents, phase, tester_ref, i);
   	    		threads_list[i].start();
   	    	}

   	    	for (int i=0; i<nthreads; i++) {
   	    		threads_list[i].join();
   	    	}

   	    	System.exit(0);
   	    	
		} catch (SystemException e) {
			System.err.println("Main: org.omg.CORBA.SystemException " + e.getMessage());
			e.printStackTrace();
			System.exit(1);
		} catch (Exception e) {
			System.err.println("Main: Unexpected exception " + e.getMessage());
			e.printStackTrace();
			System.exit(1);
		}
 	}
}

class ClientMT_Thread extends Thread {
    private ORB m_orb;
    private int m_clid;
    private int m_nevents;
    private int m_phase;
    private Tester1 m_tester_ref;
    private int m_id;

	private static final int OBJREF_BUFFER_SIZE = 1000;

    public ClientMT_Thread(ORB orb, int clid, int nevents, int phase, Tester1 tester_ref, int id)
    {
        this.m_orb = orb;
        this.m_clid = clid;
        this.m_nevents = nevents;
        this.m_phase = phase;
        this.m_tester_ref = tester_ref;
        this.m_id = id;
    }

	void terminarEjecucion (int a) {
		//this.foo_ref.shutdown();
		System.exit(0);
	}
	
	static void buildString (int n, String c) {
		c = new String();
		for (int i=0; i<n; i++)
			c = c + 'a';
	}
	
	public static long timeDiff (Date t1, Date t2) {
		long diff, mili_t1, mili_t2;
		mili_t1 = t1.getTime();
		mili_t2 = t2.getTime();
		diff = mili_t2 - mili_t1;
		return diff*1000; // Resultado en microsegundos
	}
	
	void tester () {
	    
		// Crea el fichero donde se almacenarn los resultados
		String name_result_file = "clientMT_" + m_clid + "_th" + m_id + "_ph" + m_phase + ".result";
		File file = new File(name_result_file);

		int tamano = 16; //16384
   	    String cadena = new String();
   	    buildString(tamano, cadena);
   	    	
   	    Date t1, t2;
   	    long total=0, tt;

   	    // Dato simple
   	    Any any_simple = m_orb.create_any();
   	    any_simple.insert_string(cadena);
   	    	
    	// Dato complejo
    	Any any_complex = m_orb.create_any();
    	ComplexData data = new ComplexData();
    	data.buffer = new byte[OBJREF_BUFFER_SIZE];
    	data.when = new Test.Date();
    	data.when.hour = 456;
    	data.name = cadena;
    	data.employes = new Person[30];
    	for (int i=0; i<30; i++) {
    		String cadena2 = new String();
    		buildString(3+(70*i), cadena2);
    		data.employes[i] = new Person();
    		data.employes[i].name = "Nombre1";
    		data.employes[i].last_name = "Nombre2";
    		data.employes[i].alias = cadena2;
    		data.employes[i].age = (new Integer((i*2) + 5)).shortValue();
    		data.employes[i].from = new Place();
    		data.employes[i].from.p = new Point();
    		data.employes[i].from.p.x = (new Integer(3+i)).shortValue();
    		data.employes[i].from.p.y = (new Integer(4+i)).shortValue();
    		data.employes[i].from.p.z = (new Integer(9+i)).shortValue();
    		data.employes[i].from.name = "Nombre1";
    	}
    	data.places = new Place[10];
    	for (int i=0; i<10; i++) {
    		data.places[i] = new Place();
    		data.places[i].name = "Nombre1";
    		data.places[i].p = new Point();
    	}
    	ComplexDataHelper.insert(any_complex, data);
   	    
   	    BufferedWriter bw;
		try {
			bw = new BufferedWriter(new FileWriter(file));

			switch(m_phase) {
				case 1:
					//
					// Phase 1
					//
					//System.out.println("[Phase 1]");
					String res = "[Phase 1]\n";
					bw.write(res, 0, res.length());
					m_tester_ref.register_client(m_id, m_nevents);
						
					for (int i=0; i<m_nevents; i++) {
						t1 = new Date();
						m_tester_ref.push1_phase1(any_simple);
						t2 = new Date();
						tt = timeDiff(t1, t2);
						total = total + tt;
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push1 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push1 Simple Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						t1 = new Date();
						m_tester_ref.push1_phase1(any_complex);
						t2 = new Date();
						tt = timeDiff(t1, t2);
						total = total + tt;
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push1 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push1 Complex Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						t1 = new Date();
						Any my_any = m_tester_ref.push2_phase1(any_simple);
						t2 = new Date();
						tt = timeDiff(t1, t2);
						total = total + tt;
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push2 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push2 Simple Any " + total/m_nevents);
						
					total = 0;
					for (int i=0; i<m_nevents; i++) {
						t1 = new Date();
						Any my_any = m_tester_ref.push2_phase1(any_complex);
						t2 = new Date();
						tt = timeDiff(t1, t2);
						total = total + tt;
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push2 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push2 Complex Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						t1 = new Date();
						m_tester_ref.push3_phase1(any_simple);
						t2 = new Date();
						tt = timeDiff(t1, t2);
						total = total + tt;
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push3 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push3 Simple Any " + total/m_nevents);
						
					total = 0;
					for (int i=0; i<m_nevents; i++) {
						t1 = new Date();
						m_tester_ref.push3_phase1(any_complex);
						t2 = new Date();
						tt = timeDiff(t1, t2);
						total = total + tt;
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push3 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push3 Complex Any " + total/m_nevents);

					break;
						
				case 2:
					//
					// Phase 2
					//
					//System.out.println("[Phase 2]");
					res = "[Phase 2]\n";
					bw.write(res, 0, res.length());
					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push1_phase2(any_simple);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push1 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push1 Simple Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push1_phase2(any_complex);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push1 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push1 Complex Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push2_phase2(any_simple);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push2 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push2 Simple Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push2_phase2(any_complex);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push2 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push2 Complex Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push3_phase2(any_simple);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push3 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push3 Simple Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push3_phase2(any_complex);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push3 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push3 Complex Any " + total/m_nevents);

					break;
						
				case 3:
					//
					// Phase 3
					//
					//System.out.println("[Phase 3]");
					res = "[Phase 3]\n";
					bw.write(res, 0, res.length());
					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push1_phase3(any_simple);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push1 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push1 Simple Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push1_phase3(any_complex);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push1 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push1 Complex Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push2_phase3(any_simple);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push2 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push2 Simple Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push2_phase3(any_complex);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push2 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push2 Complex Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push3_phase3(any_simple);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push3 Simple Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push3 Simple Any " + total/m_nevents);

					total = 0;
					for (int i=0; i<m_nevents; i++) {
						total = total + m_tester_ref.push3_phase3(any_complex);
					}
					res = "[Client " + m_clid + " | Thread " + m_id + "] Push3 Complex Any " + total/m_nevents + "\n";
					bw.write(res, 0, res.length());
					//System.out.println("[Client " + m_clid + " | Thread " + m_id + "] Push3 Complex Any " + total/m_nevents);

					break;
						
				case 4:
					break;
			}

			// cierra el fichero de los resultados
			bw.flush();
			bw.close();

		} catch (SystemException e) {
			System.err.println("Main: org.omg.CORBA.SystemException at phase " + m_phase + ": " + e.getMessage());
			e.printStackTrace();
		} catch (IOException e) {
			System.err.println("Main: java.io.IOException at phase " + m_phase + ": " + e.getMessage());
			e.printStackTrace();
		} catch (Exception e) {
			System.err.println("Main: Unexpected exception at phase " + m_phase + ": " + e.getMessage());
			e.printStackTrace();
		}
	}

    public void run()
    {
        tester();
    }
}
