/*
* MORFEO Project
* http://www.morfeo-project.org
*
* Component: TIDorbJTest
* Programming Language: Java
*
* File: $Source: Server.java $
* Version: $Revision: 1.0 $
* Date: $Date: 22-sep-2006 $
* Last modified by: $Author: irenka.redondo $
*
* (C) Copyright 2004 Telefnica Investigacin y Desarrollo
*     S.A.Unipersonal (Telefnica I+D)
*
* Info about members and contributors of the MORFEO project
* is available at:
*
*   http://www.morfeo-project.org/TIDorbJ/CREDITS
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*
* If you want to use this software an plan to distribute a
* proprietary application in any way, and you are not licensing and
* distributing your source code under GPL, you probably need to
* purchase a commercial license of the product.  More info about
* licensing options is available at:
*
*   http://www.morfeo-project.org/TIDorbJ/Licensing
*/

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.UserException;
import org.omg.IOP.Codec;
import org.omg.IOP.CodecFactory;
import org.omg.IOP.CodecFactoryHelper;
import org.omg.IOP.ENCODING_CDR_ENCAPS;
import org.omg.IOP.Encoding;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.POAManager;

import Test.Tester1;
import Test.Tester1Helper;
import Test.Tester2;
import Test.Tester2Helper;
import Test.Tester3;
import Test.Tester3Helper;
import Test.Tester6;
import Test.Tester6Helper;

public class Server {
	
    static Object readIOR (String file_name, ORB orb) 
        throws IOException 
    {
        String ior;
        File file = new File(file_name);

        BufferedReader br = new BufferedReader(new FileReader(file));
        ior = br.readLine();
        br.close();
        return orb.string_to_object(ior);
    }



    static void writeIOR2File (String file_name, ORB orb, Object ref) 
        throws IOException {
        
        String ior = orb.object_to_string(ref);
        File file = new File(file_name);
        
        BufferedWriter bw = new BufferedWriter(new FileWriter(file));
        bw.write(ior, 0, ior.length());
        bw.flush();
        bw.close();
    }


    public static void main(String[] args) {
        // Identificador del servidor
        String server_name = "Server1";
			
        String pwd, uri;
			
        try {
            // Crea e inicializa el orb
            /*
              Properties props = new Properties();
              props.setProperty("org.omg.CORBA.ORBClass", 
                                "es.tid.TIDorbj.core.TIDORB");
              props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                                "es.tid.TIDorbj.core.SingletonORB");
              ORB orb = ORB.init(args, props);
            */
			
            ORB orb = ORB.init(args, System.getProperties());

            // Obtiene referencias al rootPOA y su POAManager
            Object obj = orb.resolve_initial_references("RootPOA");
            POA rootpoa = POAHelper.narrow(obj);
            POAManager rootpoamgr = rootpoa.the_POAManager();
			
            // Obtiene codec factory
            obj = orb.resolve_initial_references("CodecFactory");
            CodecFactory codec_factory = CodecFactoryHelper.narrow(obj);
			
            // Crea codec
            Encoding encoding = new Encoding();
            encoding.format = ENCODING_CDR_ENCAPS.value;
            encoding.major_version = 1;
            encoding.minor_version = 2;
            Codec codec = codec_factory.create_codec(encoding);
			
            // Not supported by Visibroker
            /*
              pwd = System.getProperty("user.dir");
              uri = "file://" + pwd + "/tester3.ior";
   	    	
              Object obj3 = orb.string_to_object(uri);
            */ 

            Object obj3 = readIOR("Tester3.ior", orb);
            Tester3 tester3_ref = Tester3Helper.narrow(obj3);
   	    

            //
            // Create and activate Tester2
            //
            Tester2Impl servant2 = new Tester2Impl(server_name, orb, rootpoa, codec);
            byte[] oid2 = rootpoa.activate_object(servant2);
            Object obj2 = rootpoa.id_to_reference(oid2);
            Tester2 tester2_ref = Tester2Helper.narrow(obj2);
   		

            //
            // Create and activate Tester1
            //
            Tester1Impl servant1 = new Tester1Impl(server_name, orb, rootpoa, codec, 
                                                  20, tester2_ref, tester3_ref);
            byte[] oid1 = rootpoa.activate_object(servant1);
            Object obj1 = rootpoa.id_to_reference(oid1);
            Tester1 tester1_ref = Tester1Helper.narrow(obj1);
            servant1.set_local_reference(tester1_ref);
            writeIOR2File("Tester1.ior", orb, tester1_ref);

	
            //
            // Create and activate SurveillanceManagerAsterix30
            //
            SurveillanceManagerAsterix30Impl servantSurveillance = 
                new SurveillanceManagerAsterix30Impl();
            byte[] oid4 = rootpoa.activate_object(servantSurveillance);
            Object obj4 = rootpoa.id_to_reference(oid4);
            writeIOR2File("SurveillanceManagerAsterix30.ior", orb, obj4);
            

            //
            // Create and activate RadarAnyPing
            //
            RadarAnyPingImpl servantRadarAny = new RadarAnyPingImpl(orb);
            byte[] oid5 = rootpoa.activate_object(servantRadarAny);
            Object obj5 = rootpoa.id_to_reference(oid5);
            writeIOR2File("RadarAnyPing.ior", orb, obj5);
            

            //
            // Create and activate Tester6
            //
            Tester6Impl servant6 = new Tester6Impl(server_name, orb, rootpoa, codec);
            byte[] oid6 = rootpoa.activate_object(servant6);
            Object obj6 = rootpoa.id_to_reference(oid6);
            Tester6 tester6_ref = Tester6Helper.narrow(obj6);
            writeIOR2File("Tester6.ior", orb, tester6_ref);



            // Activa el RootPOAManager
            rootpoamgr.activate();
   			
            // Acepta peticiones
            System.out.println("[" + server_name + "] Running...");

            orb.run();
            orb.destroy();
			
            System.exit(0);

        } catch (SystemException e) {
            System.err.println("Main: org.omg.CORBA.SystemException " + 
                               e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (UserException e) {
            System.err.println("Main: org.omg.CORBA.UserException " + 
                               e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (IOException e) {
            System.err.println("Main: java.io.IOException " + 
                               e.getMessage());
            e.printStackTrace();
            System.exit(1);
        } catch (Exception e) {
            System.err.println("Main: Unexpected exception " + 
                               e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }
    }
}
