/*
 * Morfeo Project
 * http://morfeo-project.org/
 * 
 * Copyright (C) 2008 Telefonica Investigacion y Desarrollo S.A. Unipersonal
 * 
 * Author      : Alvaro Polo Valdenebro <apv at tid.es>
 * Module      : Client
 * Description : Main program for client side in Overload Test
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *     http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 *
 */

import Overload.OverloadedServer;
import Overload.OverloadedServerHelper;
import Overload.OverloadedServerPackage.Statistics;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.io.FileInputStream;

import java.util.Date;
import java.util.Properties;

import java.text.NumberFormat;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;

class Client
{
   
   public static String copyrightNote()
   {
      return 
         "Overloader Test Client for TIDorbJ\n" + 
         "Morfeo Project (http://morfeo-project.org)\n" +
         "Copyright 2008 Telefonica Investigacion y " +
         "Desarrollo S.A. Unipersonal\n\n" +
         "This software comes with ABSOLUTELY NO WARRANTY. " +
         "This is free software,\n" +
         "and you are welcome to modify and redistribute it " +
         "under the GPL license\n";
   }
   
   public static String usage()
   {
      return
"Usage, \n" + 
"\n" +
"   Client\n" +
"\n" +
"      Execute client (use file OverloadTest.properties)\n" + 
"\n" +
"   Client --server-ior <ior>\n" +
"\n" +
"      Execute client (use file OverloadTest.properties) and use\n" +
"      <ior> as server reference\n" +
"\n" +
"   Client <props file>\n" +
"\n" +
"      Execute client (use file <props file>)\n" + 
"\n" +
"   Client --server-ior <ior> <props file>\n" +
"\n" +
"      Execute client (use file <props file>) and use <ior>\n" +
"      as server reference\n" +
"\n" +
"   Client --help        : Print this help\n"; 
   }
   
   public static void singleMode(
         OverloadedServer server,
         Properties props,
         boolean verbose)
   {
      /***
       *  Some useful aliases
       */
      PrintStream err = System.err;
      PrintStream out = System.out;
      NumberFormat num_for = NumberFormat.getInstance();

      /***
       * Prepare number formating
       */
      num_for.setMaximumIntegerDigits(7);
      num_for.setMinimumIntegerDigits(7);
      num_for.setMaximumFractionDigits(2);
      num_for.setMinimumFractionDigits(2);
      
      /**
       * Extract properties values
       */
      String thr_num_prop = 
         props.getProperty("SingleMode.Threads", "2");      
      String req_num_prop = 
         props.getProperty("SingleMode.Requests", "10000");
      String flo_thr_prop =
         props.getProperty("FloodThreshold", "0.2");
      String req_apply_to_prop =
         props.getProperty("RequestApplyTo", "Global");

      
      int threads_number = 0, requests_number = 0;
      boolean req_apply_thread = false;
      double flood_threshold = 0.0f;
      try
      {
         threads_number    = Integer.parseInt(thr_num_prop);
         requests_number   = Integer.parseInt(req_num_prop);
         flood_threshold   = Double.parseDouble(flo_thr_prop);
      }
      catch (NumberFormatException e)
      {
         System.err.println("Error: invalid number format in arguments");
         System.exit(-1);
      }
      if (req_apply_to_prop.compareToIgnoreCase("Global") == 0)
         req_apply_thread = false;
      else if (req_apply_to_prop.compareToIgnoreCase("Thread") == 0)
         req_apply_thread = true;
      else
      {
         System.err.println(
            "Error: RequestApplyTo property set to an invalid value");
         System.exit(-1);
      }
      
      if (flood_threshold < 0.0f || flood_threshold > 0.5f)
      {
         System.err.println(
               "Error: flood threshold range must be between 0.0 and 0.5");
         System.exit(-1);
      }    
      
      /***
       * Create overloader threads
       */
      OverloaderThread [] threads = new OverloaderThread[threads_number];
      for (int i = 0; i < threads_number; i++)
      {
         if (req_apply_thread)
         {  
            threads[i] = new OverloaderThread(server, requests_number);
         }
         else
         {
            if (requests_number % threads_number == 0)
            {
               threads[i] = new OverloaderThread(
                     server, 
                     requests_number / threads_number);
            }
            else
            {
               if (i != threads_number - 1)
                  threads[i] = new OverloaderThread(
                        server, 
                        requests_number / (threads_number - 1));
               else
                  threads[i] = new OverloaderThread(
                        server, 
                        requests_number % (threads_number - 1));               
            }
         }
      }
      
      /***
       * Reset server
       */
      if (req_apply_thread)
         server.reset(flood_threshold, requests_number * threads_number);
      else
         server.reset(flood_threshold, requests_number);
      
      /***
       * Start the threads and wait to finish
       */
      for (int i = 0; i < threads_number; i++)
         threads[i].start();
      
      for (int i = 0; i < threads_number; i++)
      {
         try
         {
            threads[i].join();
         }
         catch (InterruptedException e)
         {
            err.println("Warning: thread was interrupted");
         }
      }
      
      /***
       * Obtain and print results
       */
      int successes = 0;
      int errors = 0;
      
      for (int i = 0; i < threads_number; i++)
      {
         successes += threads[i].getSuccesses();
         errors += threads[i].getErrors();
      }
      float rate = ((float) successes) / ((float)errors + successes);
      
      // Due to server overloading, this call may fail.
      // To avoid such fail, we insist until the server responses.
      Statistics srv_stats = null;
      while (true)
      {
         try
         {
            srv_stats = server.getStats();
            break;
         }
         catch (Exception e)
         {
         }
      }      

      if (verbose)
      {
         err.println("Done\n");
         
         err.println("Statistics");
         err.println("==========\n");
         
         err.println("  [  1 ] : Client threads");
         err.println("  [  2 ] : Requests to be sent");
         err.println("  [  3 ] : Expected requests by server");
         err.println("  [  4 ] : Arrived requests at server");
         err.println("  [  5 ] : Requests in flood threshold");
         err.println("  [  6 ] : Statistics time");
         err.println("  [  7 ] : Request rate (req/s)");
         err.println("  [  8 ] : Requests sent by client");
         err.println("  [  9 ] : Success replies at client");
         err.println("  [ 10 ] : Error replies at client");
         err.println("  [ 11 ] : Success ratio at client");
         
         err.println("");
         
         err.print(" [1] : ");
         err.print("   [2]    : ");
         err.print("   [3]    : ");
         err.print("   [4]    : ");
         err.print("   [5]    : ");
         err.print("   [6]    : ");
         err.print("     [7]     : ");
         err.print("   [8]    : ");
         err.print("   [9]    : ");
         err.print("   [10]   : ");
         err.print("  [11]   ");
         
         err.print("\n");
      }
      
      out.print(" ");
      num_for.setMinimumIntegerDigits(3);
      num_for.setMinimumFractionDigits(0);
      out.print(num_for.format(threads_number) + " : ");
      num_for.setMinimumIntegerDigits(7);
      out.print(num_for.format(requests_number) + " : ");
      out.print(num_for.format(srv_stats.expected_requests) + " : ");
      out.print(num_for.format(srv_stats.total_requests) + " : ");
      out.print(num_for.format(srv_stats.ignored_requests) + " : ");
      out.print(num_for.format(srv_stats.milliseconds) + " : ");
      num_for.setMinimumFractionDigits(2);
      out.print(num_for.format(srv_stats.requests_per_second) + " : ");
      num_for.setMinimumFractionDigits(0);
      out.print(num_for.format(
            req_apply_thread ? 
                  requests_number * threads_number : 
                  requests_number) + " : ");
      out.print(num_for.format(successes) + " : ");
      out.print(num_for.format(errors) + " : ");
      num_for.setMinimumIntegerDigits(3);
      num_for.setMinimumFractionDigits(2);
      out.print(num_for.format(rate * 100) + "%");
      
      out.println("");      
   }

   public static void multipleMode(
         OverloadedServer server,
         Properties props)
   {
      Properties new_props = new Properties(props);
      
      String thr_from_prop = 
         props.getProperty("MultipleMode.Threads.From", "2");
      String thr_to_prop = 
         props.getProperty("MultipleMode.Threads.To", "20");
      String thr_step_prop = 
         props.getProperty("MultipleMode.Threads.Step", "2");
      
      String req_from_prop = 
         props.getProperty("MultipleMode.Requests.From", "50000");
      String req_to_prop = 
         props.getProperty("MultipleMode.Requests.To", "200000");
      String req_step_prop = 
         props.getProperty("MultipleMode.Requests.Step", "50000");
      
      long thr_from = 0, thr_to = 0, thr_step = 0;
      long req_from = 0, req_to = 0, req_step = 0;
      
      try
      {
         thr_from = Long.parseLong(thr_from_prop);
         thr_to   = Long.parseLong(thr_to_prop);
         thr_step = Long.parseLong(thr_step_prop);

         req_from = Long.parseLong(req_from_prop);
         req_to   = Long.parseLong(req_to_prop);
         req_step = Long.parseLong(req_step_prop);
      }
      catch (NumberFormatException e)
      {
         System.err.println("Error: invalid number format in properties");
         System.exit(-1);
      }
      
      for (long thr = thr_from; thr <= thr_to; thr += thr_step)
         for (long req = req_from; req <= req_to; req += req_step )
         {
            new_props.setProperty("TestMode", "Single");
            new_props.setProperty("SingleMode.Threads", 
                  Long.toString(thr));
            new_props.setProperty("SingleMode.Requests", 
                  Long.toString(req));

            singleMode(server, new_props, thr == thr_from && req == req_from);
         }
   }
   
   // Main program
   public static void main(String [] args)
   {
      /***
       *  Some useful aliases
       */
      PrintStream err = System.err;
      PrintStream out = System.out;
      
      /***
       * Copyright note
       */
      err.println(
         "Overloader Test Client for TIDorbJ\n" + 
         "Morfeo Project (http://morfeo-project.org)\n" +
         "Copyright 2008 Telefonica Investigacion y " +
         "Desarrollo S.A. Unipersonal\n\n" +
         "This software comes with ABSOLUTELY NO WARRANTY. " +
         "This is free software,\n" +
         "and you are welcome to modify and redistribute it " +
         "under the GPL license\n");
      
      /***
       * Check arguments
       */
      String properties_filename = null;
      String server_ior = null;
      if (args.length > 0)
      {
         if (args[0].equals("--help"))
         {
            System.err.println(usage());
            System.exit(0);
         }
         
         if (args[0].equals("--server-ior"))
         {
            if (args.length < 2)
            {
               err.println("Error: missing server ior value");
               System.exit(-1);
            }
            server_ior = args[1];
            
            if (args.length == 3)
               properties_filename = args[2];
         }
         else
            properties_filename = args[0];
      }
      
      
      /***
       * Read properties from file
       */
      Properties props = new Properties();
      try
      {
         FileInputStream props_file = new FileInputStream(
               properties_filename == null ? 
                     "OverloadTest.properties" : properties_filename);
         props.load(props_file);
         props_file.close();
      }
      catch (java.io.IOException e)
      {
         if (properties_filename != null)
         {
            System.err.println("Error: properties file '" + properties_filename +
                  "' cannot be read\n");
            usage();
            System.exit(-1);
         }
      }
      
      String mode = props.getProperty("TestMode", "Single");
                  
      /***
       * Obtain server IOR from disk if needed
       */
      if (server_ior == null)
      {
         try
         {
            File file = new File(".OverloadTest_server.ior");
            BufferedReader br = new BufferedReader(new FileReader(file));
            server_ior = br.readLine();
            br.close();
         }
         catch (Exception e)
         {
            err.println("Error: cannot find IOR file, server running?");
            System.exit(-1);
         }
      }

      /***
       * Obtain server reference
       */
      Properties sys_props = new Properties(System.getProperties());
      
      sys_props.setProperty("org.omg.CORBA.ORBClass", 
                        "es.tid.TIDorbj.core.TIDORB");
      sys_props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                        "es.tid.TIDorbj.core.SingletonORB");
      
      ORB orb = ORB.init(args, sys_props);
      
      Object server_ref = orb.string_to_object(server_ior);
      OverloadedServer server = OverloadedServerHelper.narrow(server_ref);
      
      if (mode.compareToIgnoreCase("Single") == 0)
         singleMode(server, props, true);
      else if (mode.compareToIgnoreCase("Multiple") == 0)
         multipleMode(server, props);

      orb.destroy();

      System.exit(0);
   }
}
