/*
 * Morfeo Project
 * http://morfeo-project.org/
 * 
 * Copyright (C) 2008 Telefonica Investigacion y Desarrollo S.A. Unipersonal
 * 
 * Author      : Alvaro Polo Valdenebro <apv at tid.es>
 * Module      : OverloaderThread
 * Description : A thread class aimed to overload the server
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *     http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 *
 */

import Overload.OverloadedServer;
import Overload.StructType;
import Overload.StructTypeHolder;
import Overload.ComplexStructType;
import Overload.ComplexStructTypeHolder;

import java.util.Random;
import java.util.Date;

class OverloaderThread extends Thread
{
 
   private long m_number_of_requests;
   private long m_successes;
   private long m_errors;
   
   private Date m_start;
   private Date m_stop;
   
   private Random m_rnd;
   
   private OverloadedServer m_server;
   
   protected void callDoSomethingIn()
   {
      try
      {
         m_server.doSomethingIn(new StructType());
         m_successes++;
      }
      catch (Exception e) { m_errors ++; }
   }

   protected void callDoSomethingOut()
   {
      try
      {
         m_server.doSomethingOut(new StructTypeHolder(new StructType()));
         m_successes++;
      }
      catch (Exception e) { m_errors ++; }
   }

   protected void callDoSomethingInOut()
   {
      try
      {
         m_server.doSomethingInOut(new StructTypeHolder(new StructType()));
         m_successes++;
      }
      catch (Exception e) { m_errors ++; }
   }

   protected void callDoSomethingReturn()
   {
      try
      {
         m_server.doSomethingReturn();
         m_successes++;
      }
      catch (Exception e) { m_errors ++; }
   }

   protected void callDoSomethingComplexIn()
   {
      try
      {
         m_server.doSomethingComplexIn(new ComplexStructType());
         m_successes++;
      }
      catch (Exception e) { m_errors ++; }
   }

   protected void callDoSomethingComplexOut()
   {
      try
      {
         m_server.doSomethingComplexOut(
               new ComplexStructTypeHolder(new ComplexStructType()));
         m_successes++;
      }
      catch (Exception e) { m_errors ++; }
   }

   protected void callDoSomethingComplexInOut()
   {
      try
      {
         m_server.doSomethingComplexInOut(
               new ComplexStructTypeHolder(new ComplexStructType()));
         m_successes++;
      }
      catch (Exception e) { m_errors ++; }
   }

   protected void callDoSomethingComplexReturn()
   {
      try
      {
         m_server.doSomethingComplexReturn();
         m_successes++;
      }
      catch (Exception e) { m_errors ++; }
   }

   /* Constructor */
   public OverloaderThread(
         OverloadedServer server,
         long number_of_requests)
   {
      m_number_of_requests = number_of_requests;
      m_successes = 0;
      m_errors = 0;
      
      m_start = null;
      m_stop = null;
      
      m_rnd = new Random();
      
      m_server = server;
   }
   
   public long getSuccesses()
   {
      return m_successes;
   }
   
   public long getErrors()
   {
      return m_errors;
   }
   
   public float getSuccessRate()
   {
      return ((float) m_successes) / ((float)(m_successes + m_errors));
   }
   
   /* Thread body */
   public void run()
   {
      for (int i = 0; i < m_number_of_requests; i++)
      {
         int call = m_rnd.nextInt(8);
         
         switch (call)
         {
            case 0: callDoSomethingIn();            break;
            case 1: callDoSomethingOut();           break;
            case 2: callDoSomethingInOut();         break;
            case 3: callDoSomethingReturn();        break;
            case 4: callDoSomethingComplexIn();     break;
            case 5: callDoSomethingComplexOut();    break;
            case 6: callDoSomethingComplexInOut();  break;
            case 7: callDoSomethingComplexReturn(); break;      
            default: i--; break;
         }
      }
   }
}
