/*
 * Morfeo Project
 * http://morfeo-project.org/
 * 
 * Copyright (C) 2008 Telefonica Investigacion y Desarrollo S.A. Unipersonal
 * 
 * Author      : Alvaro Polo Valdenebro <apv at tid.es>
 * Module      : Server
 * Description : Main program for server side in Overload Test
 *
 * Info about members and contributors of the MORFEO project
 * is available at:
 *
 *     http://www.morfeo-project.org/TIDorbJ/CREDITS
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * If you want to use this software an plan to distribute a
 * proprietary application in any way, and you are not licensing and
 * distributing your source code under GPL, you probably need to
 * purchase a commercial license of the product.  More info about
 * licensing options is available at:
 *
 *   http://www.morfeo-project.org/TIDorbJ/Licensing
 *
 */

import org.omg.CORBA.ORB;
import org.omg.CORBA.Object;
import org.omg.CORBA.SystemException;
import org.omg.CORBA.UserException;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;

import java.util.Properties;

public class Server
{

   public static void main(String[] args)
   {
      try
      {
         PrintStream err = System.err;
         
         err.println(
            "Overloader Test Server for TIDorbJ\n" + 
            "Morfeo Project (http://morfeo-project.org)\n" +
            "Copyright 2008 Telefonica Investigacion y " +
            "Desarrollo S.A. Unipersonal\n\n" +
            "This software comes with ABSOLUTELY NO WARRANTY. " +
            "This is free software,\n" +
            "and you are welcome to modify and redistribute it " +
            "under the GPL license\n");
         
         // Check arguments
         boolean print_ior = false;
         if (args.length > 0)
         {
            if (args[0].compareToIgnoreCase("--print-ior") == 0)
               print_ior = true;
            else
               System.err.println(
                     "Warning: unknown argument " +
                     args[0] +
                     " ignored");
         }
         if (args.length > 1)
         {
            System.err.println("Warning: too many arguments ignored");
         }
         
         // Create and initialize orb
         Properties props = new Properties(System.getProperties());
         
         props.setProperty("org.omg.CORBA.ORBClass", 
                           "es.tid.TIDorbj.core.TIDORB");
         props.setProperty("org.omg.CORBA.ORBSingletonClass", 
                           "es.tid.TIDorbj.core.SingletonORB");
         
         ORB orb = ORB.init(args,props);

         Object poa_ref = orb.resolve_initial_references("RootPOA");
         POA root_poa = POAHelper.narrow(poa_ref);

         OverloadedServerImpl servant = new OverloadedServerImpl();
         Object ref = root_poa.servant_to_reference(servant);
         
         /***
          * Write IOR to file and stdout
          */
         String ior = orb.object_to_string(ref);
         File file = new File(".OverloadTest_server.ior");

         BufferedWriter bw = new BufferedWriter(new FileWriter(file));
         bw.write(ior, 0, ior.length());
         bw.flush();
         bw.close();
         
         if (print_ior)
            System.out.println(ior);

         root_poa.the_POAManager().activate();
         err.println("Server is running... ");
         orb.run();

         orb.destroy();

         System.exit(0);

      } catch (SystemException e)
      {
         System.err.println("Main: org.omg.CORBA.SystemException "
               + e.getMessage());
         e.printStackTrace();
         System.exit(1);
      } catch (UserException e)
      {
         System.err.println("Main: org.omg.CORBA.UserException "
               + e.getMessage());
         e.printStackTrace();
         System.exit(1);
      } catch (IOException e)
      {
         System.err.println("Main: java.io.IOException " + e.getMessage());
         e.printStackTrace();
         System.exit(1);
      } catch (Exception e)
      {
         System.err.println("Main: Unexpected exception " + e.getMessage());
         e.printStackTrace();
         System.exit(1);
      }
   }
}
